@@ -1064,6 +1064,114 @@ void sigczero_rev_setup(void)
         gensym("dsp"), A_CANT, 0);
 }
 
+/* ---------------- slop~ - slewing low-pass filter ----------------- */
+
+typedef struct slop_tilde
+{
+    t_object x_obj;
+    t_sample x_f;
+    t_sample x_coef;
+    t_sample x_last;
+    t_sample x_sigin;
+    t_sample x_freqin;
+    t_sample x_poslimitin;
+    t_sample x_posfreqin;
+    t_sample x_neglimitin;
+    t_sample x_negfreqin;
+} t_slop_tilde;
+
+t_class *slop_tilde_class;
+
+static void *slop_tilde_new(t_symbol *s, int argc, t_atom *argv)
+{
+    t_slop_tilde *x = (t_slop_tilde *)pd_new(slop_tilde_class);
+    signalinlet_new(&x->x_obj, atom_getfloatarg(0, argc, argv));
+    signalinlet_new(&x->x_obj, atom_getfloatarg(1, argc, argv));
+    signalinlet_new(&x->x_obj, atom_getfloatarg(2, argc, argv));
+    signalinlet_new(&x->x_obj, atom_getfloatarg(3, argc, argv));
+    signalinlet_new(&x->x_obj, atom_getfloatarg(4, argc, argv));
+    outlet_new(&x->x_obj, &s_signal);
+    x->x_coef = 0;
+    return (x);
+}
+
+static void slop_tilde_set(t_slop_tilde *x, t_floatarg q)
+{
+    x->x_last = q;
+}
+
+static t_int *slop_tilde_perform(t_int *w)
+{
+    t_slop_tilde *x = (t_slop_tilde *)(w[1]);
+    t_sample *sigin = (t_sample *)(w[2]);
+    t_sample *freqin = (t_sample *)(w[3]);
+    t_sample *neglimit = (t_sample *)(w[4]);
+    t_sample *negfreqin = (t_sample *)(w[5]);
+    t_sample *poslimit = (t_sample *)(w[6]);
+    t_sample *posfreqin = (t_sample *)(w[7]);
+    t_sample coef = x->x_coef;
+    t_sample *out = (t_sample *)(w[8]);
+    int n = (int)w[9];
+    int i;
+    t_sample last = x->x_last;
+    for (i = 0; i < n; i++)
+    {
+        t_sample diff = *sigin++ - last;
+        t_sample inc = *freqin++ * coef, diffinc;
+        t_sample posinc = *posfreqin++ * coef;
+        t_sample neginc = *negfreqin++ * coef;
+        t_sample maxdiff = *poslimit++;
+        t_sample mindiff = *neglimit++;
+        if (inc < 0.f)
+            inc = 0.f;
+        else if (inc > 1.f)
+            inc = 1.f;
+        if (posinc < 0.f)
+            posinc = 0.f;
+        else if (posinc > 1.f)
+            posinc = 1.f;
+        if (neginc < 0.f)
+            neginc = 0.f;
+        else if (neginc > 1.f)
+            neginc = 1.f;
+        if (maxdiff < 0)
+            maxdiff = 0;
+        if (mindiff < 0)
+            mindiff = 0;
+        if (diff > maxdiff)
+            diffinc = posinc * (diff- maxdiff) + inc * maxdiff;
+        else if (diff < -mindiff)
+            diffinc = neginc * (diff + mindiff) - inc * mindiff;
+        else diffinc = inc * diff;
+        last = *out++ = last + diffinc;
+    }
+    if (PD_BIGORSMALL(last))
+        last = 0;
+    x->x_last = last;
+    return (w+10);
+}
+
+static void slop_tilde_dsp(t_slop_tilde *x, t_signal **sp)
+{
+    x->x_coef = (2 * 3.14159) / sp[0]->s_sr;
+    dsp_add(slop_tilde_perform, 9,
+        x, sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, sp[3]->s_vec, sp[4]->s_vec,
+            sp[5]->s_vec, sp[6]->s_vec, sp[0]->s_n);
+
+}
+
+void slop_tilde_setup(void)
+{
+    slop_tilde_class = class_new(gensym("slop~"), (t_newmethod)slop_tilde_new, 0,
+        sizeof(t_slop_tilde), 0, A_GIMME, 0);
+    CLASS_MAINSIGNALIN(slop_tilde_class, t_slop_tilde, x_f);
+    class_addmethod(slop_tilde_class, (t_method)slop_tilde_dsp,
+        gensym("dsp"), A_CANT, 0);
+    class_addmethod(slop_tilde_class, (t_method)slop_tilde_set, gensym("set"),
+        A_FLOAT, 0);
+}
+
+
 /* ------------------------ setup routine ------------------------- */
 
 void d_filter_setup(void)
@@ -1079,4 +1187,5 @@ void d_filter_setup(void)
     sigcpole_setup();
     sigczero_setup();
     sigczero_rev_setup();
+    slop_tilde_setup();
 }
