@@ -6,11 +6,15 @@
 away before the panel does... */
 
 #include "m_pd.h"
+#include "g_canvas.h"
 #include <stdio.h>
 #include <string.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#ifdef _MSC_VER
+#define snprintf _snprintf  /* for pdcontrol object */
+#endif
 
 /* --------------------- graphics responder  ---------------- */
 
@@ -190,7 +194,7 @@ typedef struct _openpanel
     t_symbol *x_s;
 } t_openpanel;
 
-static void *openpanel_new( void)
+static void *openpanel_new(void)
 {
     char buf[50];
     t_openpanel *x = (t_openpanel *)pd_new(openpanel_class);
@@ -245,7 +249,7 @@ typedef struct _savepanel
     t_symbol *x_s;
 } t_savepanel;
 
-static void *savepanel_new( void)
+static void *savepanel_new(void)
 {
     char buf[50];
     t_savepanel *x = (t_savepanel *)pd_new(savepanel_class);
@@ -298,7 +302,7 @@ typedef struct _key
     t_object x_obj;
 } t_key;
 
-static void *key_new( void)
+static void *key_new(void)
 {
     t_key *x = (t_key *)pd_new(key_class);
     outlet_new(&x->x_obj, &s_float);
@@ -321,7 +325,7 @@ typedef struct _keyup
     t_object x_obj;
 } t_keyup;
 
-static void *keyup_new( void)
+static void *keyup_new(void)
 {
     t_keyup *x = (t_keyup *)pd_new(keyup_class);
     outlet_new(&x->x_obj, &s_float);
@@ -346,7 +350,7 @@ typedef struct _keyname
     t_outlet *x_outlet2;
 } t_keyname;
 
-static void *keyname_new( void)
+static void *keyname_new(void)
 {
     t_keyname *x = (t_keyname *)pd_new(keyname_class);
     x->x_outlet1 = outlet_new(&x->x_obj, &s_float);
@@ -386,6 +390,98 @@ static void key_setup(void)
     class_sethelpsymbol(keyname_class, gensym("key"));
 }
 
+/* ------------------------ pdcontrol --------------------------------- */
+
+static t_class *pdcontrol_class;
+
+typedef struct _pdcontrol
+{
+    t_object x_obj;
+    t_canvas *x_canvas;
+    t_outlet *x_outlet;
+} t_pdcontrol;
+
+static void *pdcontrol_new( void)
+{
+    t_pdcontrol *x = (t_pdcontrol *)pd_new(pdcontrol_class);
+    x->x_canvas = canvas_getcurrent();
+    x->x_outlet = outlet_new(&x->x_obj, 0);
+    return (x);
+}
+
+    /* output containing directory of patch.  optional args:
+    1. a number, zero for this patch, one for the parent, etc.;
+    2. a symbol to concatenate onto the directory; */
+
+static void pdcontrol_dir(t_pdcontrol *x, t_symbol *s, t_floatarg f)
+{
+    t_canvas *c = x->x_canvas;
+    int i;
+    for (i = 0; i < (int)f; i++)
+    {
+        while (!c->gl_env)  /* back up to containing canvas or abstraction */
+            c = c->gl_owner;
+        if (c->gl_owner)    /* back up one more into an owner if any */
+            c = c->gl_owner;
+    }
+    if (*s->s_name)
+    {
+        char buf[MAXPDSTRING];
+        snprintf(buf, MAXPDSTRING, "%s/%s",
+            canvas_getdir(c)->s_name, s->s_name);
+        buf[MAXPDSTRING-1] = 0;
+        outlet_symbol(x->x_outlet, gensym(buf));
+    }
+    else outlet_symbol(x->x_outlet, canvas_getdir(c));
+}
+
+static void pdcontrol_args(t_pdcontrol *x, t_floatarg f)
+{
+    t_canvas *c = x->x_canvas;
+    int i;
+    int argc;
+    t_atom *argv;
+    for (i = 0; i < (int)f; i++)
+    {
+        while (!c->gl_env)  /* back up to containing canvas or abstraction */
+            c = c->gl_owner;
+        if (c->gl_owner)    /* back up one more into an owner if any */
+            c = c->gl_owner;
+    }
+    canvas_setcurrent(c);
+    canvas_getargs(&argc, &argv);
+    canvas_unsetcurrent(c);
+    outlet_list(x->x_outlet, &s_list, argc, argv);
+}
+
+static void pdcontrol_browse(t_pdcontrol *x, t_symbol *s)
+{
+    char buf[MAXPDSTRING];
+    snprintf(buf, MAXPDSTRING, "::pd_menucommands::menu_openfile %s\n",
+        s->s_name);
+    buf[MAXPDSTRING-1] = 0;
+    sys_gui(buf);
+}
+
+static void pdcontrol_isvisible(t_pdcontrol *x)
+{
+    outlet_float(x->x_outlet, glist_isvisible(x->x_canvas));
+}
+
+static void pdcontrol_setup(void)
+{
+    pdcontrol_class = class_new(gensym("pdcontrol"),
+        (t_newmethod)pdcontrol_new, 0, sizeof(t_pdcontrol), 0, 0);
+    class_addmethod(pdcontrol_class, (t_method)pdcontrol_dir,
+        gensym("dir"), A_DEFFLOAT, A_DEFSYMBOL, 0);
+    class_addmethod(pdcontrol_class, (t_method)pdcontrol_args,
+        gensym("args"), A_DEFFLOAT, 0);
+    class_addmethod(pdcontrol_class, (t_method)pdcontrol_browse,
+        gensym("browse"), A_SYMBOL, 0);
+    class_addmethod(pdcontrol_class, (t_method)pdcontrol_isvisible,
+        gensym("isvisible"), 0);
+}
+
 /* -------------------------- setup routine ------------------------------ */
 
 void x_gui_setup(void)
@@ -394,4 +490,5 @@ void x_gui_setup(void)
     openpanel_setup();
     savepanel_setup();
     key_setup();
+    pdcontrol_setup();
 }
